// Server.cpp (Test Server)

#include "StdAfx.h"

#include "Server.h"

using namespace System::Runtime::Remoting;
using namespace System::Runtime::Remoting::Channels;
using namespace System::Runtime::Remoting::Activation;
using namespace System::Runtime::CompilerServices;
using namespace System::Diagnostics;
using namespace System::Reflection;

using namespace TestServer;

// Server class members

int Server::GetClientInstanceNumber()
{
  return Interlocked::Increment(m_nClientInstance);
}

bool Server::IsDebugBuild()
{
#ifdef _DEBUG
  return true;
#else
  return false;
#endif
}

[MethodImpl(MethodImplOptions::Synchronized)]
String ^Server::GetSharedData()
{
  if (s_dltStatusReportCallback) s_dltStatusReportCallback(Thread::CurrentThread->ManagedThreadId);
  return m_strSharedData;
}

[MethodImpl(MethodImplOptions::Synchronized)]
void Server::SetSharedData(String ^strData)
{
  if (s_dltStatusReportCallback) s_dltStatusReportCallback(Thread::CurrentThread->ManagedThreadId);
  m_strSharedData = strData;
}

[MethodImpl(MethodImplOptions::Synchronized)]
Server ^Server::GetInstance()
{
  if (!s_mtxInstanceMutex) {
    // This means we're not the primary .exe instance
    try {
      Mutex ^mtx = Mutex::OpenExisting("TestServerInstanceMutex");
      mtx->Close();
    }
    catch (WaitHandleCannotBeOpenedException ^) {
      // No primary instance is running yet, so start one up
      Process ^proc = Process::Start(Assembly::GetExecutingAssembly()->Location);
      Trace::Assert(proc != nullptr, "TestServer: Failed to launch primary .exe instance");
      // Trace::Assert(proc->WaitForInputIdle(10000), "TestServer: Wait for idle state of launched primary .exe instance timed out");
      proc->Close();
    }
    if (!s_clientchannel) {
      s_clientchannel = gcnew TcpClientChannel;
      ChannelServices::RegisterChannel(s_clientchannel, false);
    }
    Factory ^facRemoteFactory = safe_cast<Factory ^>(Activator::CreateInstance(nullptr, "TestServer.Factory",
      gcnew array<Object ^>{gcnew UrlAttribute("tcp://localhost:55300/TestServer")})->Unwrap());
    return facRemoteFactory->GetRemoteServer();
  }
  if (!s_instance) s_instance = gcnew Server;
  return s_instance;
}

// To be called upon start-up of the supposedly singular primary server .exe instance,
// returns true if no other instance is running yet and the instance mutex was acquired:

bool Server::Startup(Action<int> ^dltStatusReportCallback)
{
  bool bIMTheOne;
  Mutex ^mtx = gcnew Mutex(true, "TestServerInstanceMutex", bIMTheOne);
  if (!bIMTheOne) {
    mtx->Close();
    return false;  // Start-up failed: primary instance already running
  }
  s_mtxInstanceMutex = mtx;
  s_dltStatusReportCallback = dltStatusReportCallback;
  ChannelServices::RegisterChannel(gcnew TcpServerChannel(55300), false);
  RemotingConfiguration::ApplicationName = "TestServer";
  RemotingConfiguration::RegisterActivatedServiceType(Factory::typeid);
  return true;
}

// To be called upon shutdown of the singular server .exe instance:

void Server::Shutdown()
{
  if (s_mtxInstanceMutex) {
    s_mtxInstanceMutex->ReleaseMutex();
    s_mtxInstanceMutex->Close();
  }
}

// Factory class members

Server ^Factory::GetServer()
{
  return Server::GetInstance();
}

Server ^Factory::GetRemoteServer()
{
  return GetServer();
}
